# @Coding: UTF-8
# @Time: 2024/9/10 13:42
# @Author: xieyang_ls
# @Filename: __init__.py
import json

from datetime import datetime

from pyutils_spirit.util.assemble import Assemble, HashAssemble

from pyutils_spirit.util.json_util import deep_dumps, deep_loads

from pyutils_spirit.util.cities import get_provinces, get_cities

from pyutils_spirit.util.set import Set, HashSet

__all__ = ['Assemble',
           'HashAssemble',
           'deep_dumps',
           'deep_loads',
           'get_provinces',
           'get_cities',
           'Set',
           'HashSet']



# @Coding: UTF-8
# @Time: 2024/9/20 12:29
# @Author: xieyang_ls
# @Filename: websocket.py

import time

import socket

import base64

import hashlib

import inspect

from abc import ABC
from typing import Any

from pyutils_spirit.style import draw_websocket_banner

from logging import info, basicConfig, INFO, exception

from pyutils_spirit.concurrent_thread.thread_holder import ThreadHolder

from pyutils_spirit.concurrent_thread.thread_executor import ThreadExecutor

basicConfig(level=INFO)


class WebSocket(ABC):
    __socket_server: socket.socket = None

    __listener_count = None

    __buffer_size: int = None

    __websocket_end_points: set[type] = None

    __executor: ThreadExecutor = None

    container: ThreadHolder[str, Any] = None

    __close_signature = None

    @classmethod
    def __start_server(cls, host, port, other_cls):
        cls.__socket_server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        cls.__listener_count = 100000
        cls.__buffer_size = 4096
        cls.__websocket_server_class = other_cls
        cls.__executor = ThreadExecutor(executor_count=cls.__listener_count + 1)
        cls.container = ThreadHolder.constructor()
        cls.__socket_server.bind((host, port))
        cls.__socket_server.listen(cls.__listener_count)
        cls.__executor.execute(cls.__listener_connection)
        cls.__close_signature = ""
        info(f"WebSocket Server listening on ws://{host}:{port}")

    class Session:
        __socket: socket.socket = None

        __address: tuple[str, int] = None

        __buffer_size: int = None

        __is_connected: bool = None

        __headers: dict = None

        __params: dict = None

        __start_connected_timer: float = None

        __close_connected_timer: float = None

        def __init__(self,
                     socket_client: socket.socket,
                     address: tuple[str, int],
                     headers: dict,
                     params: dict,
                     is_connected: bool = True,
                     buffer_size: int = 4096):
            self.__socket = socket_client
            self.__address = address
            self.__headers = headers
            self.__params = params
            self.__is_connected = is_connected
            self.__buffer_size = buffer_size
            self.__start_connected_timer = time.time()

        def get_socket_client(self) -> socket.socket:
            return self.__socket

        def get_is_connected(self) -> bool:
            return self.__is_connected

        def get_address(self):
            if self.__address is None:
                self.__address = self.__socket.getpeername()
            return self.__address

        def get_headers(self) -> dict:
            if self.__headers is not None:
                return self.__headers  # 已缓存的 headers 则直接返回

        def get_params(self) -> dict:
            return self.__params

        def send_message(self, message: str, encoding: str = "utf-8", opcode: int = 0x1) -> None:
            try:
                # 验证操作码
                if opcode not in (0x1, 0x2):
                    raise ValueError("Opcode must be 0x1 (text) or 0x2 (binary)")
                # 编码消息
                message_bytes = message.encode(encoding)
                fin = 0x80  # FIN=1（不分片）
                mask_bit = 0x00  # 服务器必须不掩码
                # 构造帧头
                header = bytearray()
                payload_len = len(message_bytes)
                # 处理长度字段
                if payload_len <= 125:
                    header += bytes([fin | opcode, payload_len | mask_bit])
                elif payload_len <= 65535:
                    header += bytes([fin | opcode, 126 | mask_bit])
                    header += payload_len.to_bytes(length=2, byteorder='big', signed=False)
                else:
                    header += bytes([fin | opcode, 127 | mask_bit])
                    header += payload_len.to_bytes(length=8, byteorder='big', signed=False)
                # 发送完整帧
                self.__socket.send(header + message_bytes)
            except (UnicodeEncodeError, ValueError) as e:
                exception(f"Encoding/Validation error: {str(e)}")
            except Exception as e:
                exception(f"Send error: {str(e)}")

        def __send_close_frame(self, code: int, reason: str) -> None:
            payload = code.to_bytes(length=2, byteorder='big', signed=False)
            if reason:
                payload += reason.encode('utf-8')
            # 构造帧头
            fin = 0x80
            opcode = 0x08  # 关闭帧
            length = len(payload)
            if length <= 125:
                frame = bytes([fin | opcode, length]) + payload
            elif length <= 65535:
                frame = bytes([fin | opcode, 126]) + length.to_bytes(length=2, byteorder='big') + payload
            else:
                frame = bytes([fin | opcode, 127]) + length.to_bytes(length=8, byteorder='big') + payload
            self.__socket.send(frame)

        def close(self, close_code: int = 1000, reason: str = "See you again~~") -> None:
            if self.__is_connected is True:
                self.__is_connected = False
                self.__close_connected_timer = time.time()
                info(f"Disconnecting Session: {self}")
                self.__send_close_frame(code=close_code, reason=reason)
                self.__socket.close()
                __endpoint = WebSocket.container.remove_holder(key=str(self.__address))
                __onclose: callable = __endpoint.ws_funcs["onclose"]
                if __onclose is not None:
                    __onclose(self=__endpoint, session=self)
                session_id = id(self.__socket)
                if session_id in __endpoint.message_buffer:
                    del __endpoint.message_buffer[session_id]
                if self.__socket in __endpoint.partial_data:
                    del __endpoint.partial_data[self.__socket]

        def __str__(self) -> str:
            if self.__is_connected:
                connected_timer: str = f"current_connected_timer={time.time() - self.__start_connected_timer}"
            else:
                connected_timer = f"total_connected_timer={self.__close_connected_timer - self.__start_connected_timer}"
            return ("{" + f"socket={self.__socket}, " +
                    f"address={self.__address}, " +
                    f"{connected_timer}" + "}")

    @classmethod
    def __handshaking(cls, socket_client: socket.socket, ws_headers: dict) -> bool:
        # WebSocket 握手
        try:
            # 检查必要字段是否存在
            ws_key = ws_headers.get("Sec-WebSocket-Key", "")
            if not ws_key:
                ws_key = ws_headers.get("sec-websocket-key", "")
                if not ws_key:
                    raise ValueError("Missing Sec-WebSocket-Key")

            # 验证 WebSocket 版本
            ws_version = ws_headers.get("Sec-WebSocket-Version", "13")
            if ws_version != "13":
                response = (
                    "HTTP/1.1 426 Upgrade Required\r\n"
                    "Sec-WebSocket-Version: 13\r\n\r\n"
                )
                socket_client.send(response.encode())
                return False

            # 生成 Accept Key
            magic_guid = "258EAFA5-E914-47DA-95CA-C5AB0DC85B11"
            accept_key = base64.b64encode(
                hashlib.sha1((ws_key + magic_guid).encode()).digest()
            ).decode()

            # 构建响应头（增加版本确认）
            response_headers = [
                "HTTP/1.1 101 Switching Protocols",
                "Upgrade: websocket",
                "Connection: Upgrade",
                f"Sec-WebSocket-Accept: {accept_key}",
                "Sec-WebSocket-Version: 13",  # 明确返回支持的版本
            ]

            # 发送握手响应
            response = "\r\n".join(response_headers) + "\r\n\r\n"
            socket_client.send(response.encode())
            return True

        except (KeyError, ValueError) as e:
            # 处理无效请求
            exception(f"Key / Value Error: {e}")
            error_response = "HTTP/1.1 400 Bad Request\r\n\r\n"
            socket_client.send(error_response.encode())
            return False

        except Exception as e:
            # 处理其他异常（如连接中断）
            exception(f"Handshake error: {str(e)}")
            return False

    @classmethod
    def __listener_connection(cls) -> None:
        while True:
            try:
                socket_client, addr = cls.__socket_server.accept()
                # 缓冲区分块接收，直到遇到 \r\n\r\n
                header_data: bytes | None = None

                buffer = bytearray()
                while True:
                    chunk = socket_client.recv(cls.__buffer_size)  # 每次接收 4KB
                    if not chunk:  # 连接关闭
                        break
                    buffer.extend(chunk)
                    if b"\r\n\r\n" in buffer:
                        # 找到头部结束位置并截断
                        header_end = buffer.index(b"\r\n\r\n")
                        header_data = buffer[:header_end]
                        break
                try:
                    # 解码并解析头部
                    ws_endpoint_cls: type | None = None
                    ws_headers: dict[str, str] = {}
                    params: dict[str, str | int] = {}
                    header_str = header_data.decode("utf-8")
                    lines = header_str.split("\r\n")
                    for end_point_cls in cls.__websocket_end_points:
                        signature = getattr(end_point_cls, "__decorator_signature__")
                        paths = lines[0].split(signature)
                        if len(paths) == 2 and paths[0][-1] == " " and paths[1][0] in (" ", "?"):
                            ws_endpoint_cls = end_point_cls
                            break
                    if ws_endpoint_cls is None:
                        ex_msg = f"Error: WebSocket URL({lines[0].split()[1]}) cannot match endpoint signature"
                        ex_str = (
                            "HTTP/1.1 400 Bad Request\r\n"
                            "Content-Type: text/plain\r\n"
                            "Connection: close({})\r\n\r\n"
                        ).format(ex_msg)
                        socket_client.send(ex_str.encode("utf-8"))
                        socket_client.close()
                        raise (ValueError(f"{ex_msg}"))
                    arguments = lines[0].split()[1].split("?")
                    if len(arguments) > 1:
                        for param in arguments[1].split("&"):
                            key, value = param.split(sep="=", maxsplit=1)
                            params[key] = value
                    for line in lines[1:]:  # 跳过请求行（如 GET / HTTP/1.1）
                        if ": " in line:
                            key, value = line.split(sep=": ", maxsplit=1)
                            ws_headers[key] = value
                except UnicodeDecodeError as e:
                    # 处理编码错误
                    exception(f"Header decoding error: {e}")
                    continue
                except ValueError as e:
                    # 处理键值对解析错误
                    exception(f"Invalid header format: {e}")
                    continue
                if cls.__handshaking(socket_client=socket_client, ws_headers=ws_headers):
                    __endpoint = ws_endpoint_cls()
                    cls.container.set_holder(key=str(addr), value=__endpoint)
                    session = cls.Session(socket_client=socket_client,
                                          address=addr,
                                          headers=ws_headers,
                                          params=params)
                    info(f"Session: {session} connection successful")
                    cls.__executor.execute(task=cls.__listener_message, end_point=__endpoint, session=session)
                    __onopen: callable = __endpoint.ws_funcs["onopen"]
                    if __onopen is not None:
                        __onopen(self=__endpoint, session=session)
            except Exception as e:
                exception(e)

    @classmethod
    def __receive_message(cls, end_point, session: Session):
        # 接收消息
        socket_client: socket.socket = session.get_socket_client()
        session_id = id(socket_client)
        data = end_point.partial_data.get(socket_client, b'') + socket_client.recv(cls.__buffer_size)

        try:
            while len(data) >= 2:
                # 1. 解析帧头
                fin = (data[0] & 0x80) >> 7
                rsv = (data[0] & 0x70) >> 4
                opcode = data[0] & 0x0F
                masked = (data[1] & 0x80) >> 7
                payload_len = data[1] & 0x7F
                # 2. 协议合规性校验
                if rsv != 0:
                    raise ValueError("RSV位违规")
                if not masked:  # 所有客户端帧必须掩码
                    raise ValueError("未掩码数据帧")
                # 3. 解析payload长度和掩码
                mask_start = 2
                if payload_len == 126:
                    if len(data) < 4:
                        break
                    payload_len = int.from_bytes(data[2:4], 'big')
                    mask_start = 4
                elif payload_len == 127:
                    if len(data) < 10:
                        break
                    if (data[2] & 0x80) != 0:
                        raise ValueError("无效长度")
                    payload_len = int.from_bytes(data[2:10], 'big')
                    mask_start = 10
                # 4. 检查数据完整性
                if len(data) < mask_start + 4 + payload_len:
                    break
                # 5. 解码payload
                masks = data[mask_start:mask_start + 4]
                payload = data[mask_start + 4:mask_start + 4 + payload_len]
                decoded = bytes([payload[i] ^ masks[i % 4] for i in range(payload_len)])
                # 6. 处理控制帧
                if opcode in (0x8, 0x9, 0xA):
                    if not fin:
                        raise ValueError("控制帧不能分片")
                    if payload_len > 125:
                        raise ValueError("控制帧负载过长")
                    if opcode == 0x8:  # Close帧
                        return session.close()
                    elif opcode == 0x9:  # Ping帧
                        # 发送Pong帧，携带解码后的数据
                        header = bytearray([0x8A])  # FIN=1, opcode=0xA
                        if len(decoded) <= 125:
                            header.append(len(decoded))
                        elif len(decoded) <= 0xFFFF:
                            header.extend([126, *len(decoded).to_bytes(length=2, byteorder='big')])
                        else:
                            header.extend([127, *len(decoded).to_bytes(length=8, byteorder='big')])
                        socket_client.send(bytes(header) + decoded)
                        # 继续处理剩余数据
                        data = data[mask_start + 4 + payload_len:]
                        continue
                    elif opcode == 0xA:  # Pong帧，忽略
                        data = data[mask_start + 4 + payload_len:]
                        continue
                # 7. 处理数据帧
                if session_id not in end_point.message_buffer:
                    end_point.message_buffer[session_id] = {'fragments': [], 'opcode': opcode}
                else:
                    if opcode != 0x0:
                        raise ValueError("分片连续性错误")
                end_point.message_buffer[session_id]['fragments'].append(decoded)
                if fin:
                    full_data = b''.join(end_point.message_buffer[session_id]['fragments'])
                    original_opcode = end_point.message_buffer[session_id]['opcode']
                    del end_point.message_buffer[session_id]
                    # 处理数据
                    if original_opcode == 0x1:
                        try:
                            result = full_data.decode('utf-8')
                        except UnicodeDecodeError as e:
                            return session.close(close_code=60001, reason=str(e))
                    else:
                        result = full_data
                    data = data[mask_start + 4 + payload_len:]
                    end_point.partial_data[socket_client] = data
                    return result
                else:
                    data = data[mask_start + 4 + payload_len:]
        except (ConnectionResetError, ValueError) as e:
            exception(f"协议错误: {e}")
            session.close(close_code=60001, reason=str(e))
        finally:
            end_point.partial_data[socket_client] = data
        return None

    @classmethod
    def __listener_message(cls, end_point, session: Session) -> None:
        while session.get_is_connected():
            try:
                message: str = cls.__receive_message(end_point=end_point, session=session)
                if message is None:
                    continue
                elif message == "ping":
                    session.send_message("pong")
                elif message == cls.__close_signature:
                    session.close()
                else:
                    __onmessage: callable = end_point.ws_funcs["onmessage"]
                    if __onmessage is not None:
                        __onmessage(self=end_point, message=message)
            except Exception as e:
                exception(e)
                __onerror: callable = end_point.ws_funcs["onerror"]
                if __onerror is not None:
                    try:
                        __onerror(self=end_point, session=session, ex=e)
                    except Exception as e:
                        exception(e)

    @classmethod
    def websocket_server(cls, host: str, port: int) -> callable:

        def decorator_handler(other_cls) -> type:
            draw_websocket_banner()
            cls.__start_server(host, port, other_cls)
            other_cls.__decorator__ = "WebSocketServer"
            other_cls.__decorator_params__ = {
                "host": host,
                "port": port
            }
            return other_cls

        return decorator_handler

    @classmethod
    def endpoint(cls, signature: str) -> callable:
        if not isinstance(signature, str):
            raise TypeError("the signature must be a string")

        def decorator_handler(other_cls) -> type:
            other_cls.__decorator__ = "WebSocketServerEndPoint"
            other_cls.__decorator_signature__ = signature
            other_cls.partial_data = {}
            other_cls.message_buffer = {}
            ws_funcs: dict[str, callable] = {
                "onopen": None,
                "onmessage": None,
                "onerror": None,
                "onclose": None
            }
            for method_name, method in inspect.getmembers(other_cls):
                decorator = getattr(method, "__decorator__", None)
                if decorator in ["onopen", "onmessage", "onerror", "onclose"]:
                    ws_funcs[decorator] = method
            other_cls.ws_funcs = ws_funcs
            if cls.__websocket_end_points is None:
                cls.__websocket_end_points = set()
            cls.__websocket_end_points.add(other_cls)
            return other_cls

        return decorator_handler

    @classmethod
    def onopen(cls) -> callable:
        def decorator_handler(func) -> None:
            func.__decorator__ = "onopen"
            cls.__onopen_func = func
            return func

        return decorator_handler

    @classmethod
    def onmessage(cls) -> callable:
        def decorator_handler(func) -> None:
            func.__decorator__ = "onmessage"
            cls.__onmessage_func = func
            return func

        return decorator_handler

    @classmethod
    def onclose(cls) -> callable:
        def decorator_handler(func) -> None:
            func.__decorator__ = "onclose"
            cls.__onclose_func = func
            return func

        return decorator_handler

    @classmethod
    def onerror(cls) -> callable:
        def decorator_handler(func) -> None:
            func.__decorator__ = "onerror"
            cls.__onerror_func = func
            return func

        return decorator_handler


websocket_server = WebSocket.websocket_server
endpoint = WebSocket.endpoint
Session = WebSocket.Session
onopen = WebSocket.onopen
onmessage = WebSocket.onmessage
onclose = WebSocket.onclose
onerror = WebSocket.onerror



from pyutils_spirit.spirit_container.annotation import component

from pyutils_spirit.util.cities import get_provinces, get_cities


@component(signature="cities")
class CitiesComponent:

    def __init__(self):
        self.get_cities: callable = get_cities
        self.get_provinces: callable = get_provinces


print("Cities Component Initialized")



import json

from datetime import datetime


def deep_dumps(data: (dict, set, tuple, list)) -> str:
    if not isinstance(data, (dict, set, tuple, list)):
        raise TypeError('the argument(data) should be a dict/set/tuple/list')

    def dumps_datetime(data: (dict, set, tuple, list)) -> (dict, set, tuple, list):
        if isinstance(data, dict):
            for key in data.keys():
                value = data[key]
                if isinstance(value, datetime):
                    data[key] = value.isoformat()
                else:
                    data[key] = dumps_datetime(data=value)
            return data
        elif isinstance(data, (set, tuple, list)):
            container: list = []
            container_type: type = type(data)
            for item in data:
                if isinstance(item, datetime):
                    container.append(item.isoformat())
                else:
                    container.append(dumps_datetime(data=item))
            return container_type(container)
        else:
            return data

    result = dumps_datetime(data=data)
    return json.dumps(obj=result, ensure_ascii=False)


def deep_loads(data: (dict, set, tuple, list, str)) -> (dict, set, tuple, list):
    if not isinstance(data, (dict, set, tuple, list, str)):
        raise TypeError('the argument(data) should be a dict/set/tuple/list/str')

    if isinstance(data, str):
        load_data: (dict, set, tuple, list) = None
        try:
            try:
                load_data = json.loads(data)
                if str(load_data) == data:
                    return data
            except json.decoder.JSONDecodeError:
                return data
            return deep_loads(data=load_data)
        except json.decoder.JSONDecodeError:
            return load_data
    elif isinstance(data, dict):
        for key in data.keys():
            value = data[key]
            if isinstance(value, (dict, set, tuple, list, str)):
                data[key] = deep_loads(data=value)
        return data
    elif isinstance(data, (set, tuple, list)):
        container: list = []
        container_type: type = type(data)
        for item in data:
            if isinstance(item, (dict, set, tuple, list, str)):
                container.append(deep_loads(data=item))
            else:
                container.append(item)
        return container_type(container)



# @Coding: UTF-8
# @Time: 2024/9/12 23:44
# @Author: xieyang_ls
# @Filename: lock.py
import time

from abc import ABC, abstractmethod

from threading import Lock, Condition, current_thread

from pyutils_spirit.util.assemble import Assemble, HashAssemble

from pyutils_spirit.exception.exception import InvalidThreadWaitError


class Regional(ABC):

    @abstractmethod
    def block(self, timeout: int = None):
        pass

    @abstractmethod
    def wake(self):
        pass

    @abstractmethod
    def wake_all(self):
        pass


class ReentryLock(object):
    __get_count: int = None

    __current_thread_id: int | None = None

    __lock: Lock = None

    __condition: Condition = None

    __assemble: Assemble = None

    def __init__(self):
        self.__get_count = 0
        self.__current_thread_id = None
        self.__lock = Lock()
        self.__condition = Condition(self.__lock)
        self.__assemble: Assemble[int, int] = HashAssemble()

    def try_lock(self) -> None:
        identity = current_thread().ident
        try:
            while True:
                self.__lock.acquire()
                if self.__current_thread_id is None:
                    self.__current_thread_id = identity
                    self.__get_count = 1
                    return None
                elif self.__current_thread_id == identity:
                    self.__get_count += 1
                    return None
                else:
                    self.__condition.wait()
                    self.__lock.release()
        finally:
            self.__lock.release()

    def release(self) -> None:
        if self.__current_thread_id == current_thread().ident:
            self.__get_count -= 1
            if self.__get_count == 0:
                self.__current_thread_id = None
                self.__lock.acquire()
                self.__condition.notify_all()
                self.__lock.release()

    def getReentryCount(self) -> int:
        return self.__get_count

    def __lock_give_up(self) -> None:
        if self.__current_thread_id == current_thread().ident:
            self.__assemble.put(self.__current_thread_id, self.__get_count)
            self.__lock.acquire()
            self.__current_thread_id = None
            self.__condition.notify_all()
            self.__lock.release()
        else:
            raise InvalidThreadWaitError

    def __try_lock_after_block(self) -> None:
        identity = current_thread().ident
        while True:
            self.__lock.acquire()
            if self.__current_thread_id is None:
                self.__current_thread_id = identity
                self.__get_count = self.__assemble.get(identity)
                self.__lock.release()
                return None
            self.__condition.wait()
            self.__lock.release()

    def initBlockRegional(self) -> Regional:
        return ReentryLock.BlockRegional(self.__lock_give_up, self.__try_lock_after_block)

    class BlockRegional(Regional):

        __lock: Lock = None

        __condition: Condition = None

        __lock_give_up: callable = None

        __try_lock_after_block: callable = None

        def __init__(self, lock_give_up, try_lock_after_block) -> None:
            self.__lock = Lock()
            self.__condition = Condition(self.__lock)
            self.__lock_give_up = lock_give_up
            self.__try_lock_after_block = try_lock_after_block

        def block(self, timeout: int = None) -> bool:
            endTime = time.time() + (timeout if timeout is not None else 0)
            self.__lock.acquire()
            self.__lock_give_up()
            self.__condition.wait(timeout=timeout)
            self.__lock.release()
            flag: bool = time.time() >= endTime
            self.__try_lock_after_block()
            return flag

        def wake(self):
            self.__lock.acquire()
            self.__condition.notify()
            self.__lock.release()

        def wake_all(self):
            self.__lock.acquire()
            self.__condition.notify_all()
            self.__lock.release()



import threading

from datetime import datetime, timezone

from pyutils_spirit.annotation.interpreter import singleton


@singleton(signature="spirit_id")
class SpiritID:
    __instance = None  # 单例模式实现
    __lock = threading.Lock()  # 类级别锁用于单例创建

    # 初始化方法设为私有
    def __init__(self):
        self.__BEGIN_TIMESTAMP = 1695600000  # 开始时间戳
        self.__COUNT_BIT = 32  # 位移位数
        self.__incr_count = 0
        self.__instance_lock = threading.Lock()  # 实例级别锁用于计数器操作

    def __get_all_seconds(self):
        """获取当前UTC时间与基准时间的秒差"""
        now = datetime.now(timezone.utc)
        return int(now.timestamp()) - self.__BEGIN_TIMESTAMP  # 精确到秒的时间差[4](@ref)

    def next(self, prefix):
        """生成带前缀的ID"""
        timestamp_part = (self.__get_all_seconds())

        with self.__instance_lock:  # 保证原子操作[4](@ref)
            self.__incr_count += 1
            if self.__incr_count >= (1 << self.__COUNT_BIT):  # 处理32位溢出
                raise OverflowError("Sequence number overflow")

            combined = (timestamp_part << self.__COUNT_BIT) | self.__incr_count
            return f"{prefix}{combined}"



# @Coding: UTF-8
# @Time: 2024/9/14 21:51
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.annotation.interpreter import connection, get_instance_signature, singleton

__all__ = ['connection',
           'get_instance_signature',
           'singleton']

print("annotation module loaded")



__provinces: list[str] = ['直辖市', '特别行政区', '河北省', '山西省',
                          '内蒙古自治区', '辽宁省', '吉林省', '黑龙江省',
                          '江苏省', '浙江省', '安徽省', '福建省',
                          '江西省', '山东省', '河南省', '湖北省',
                          '湖南省', '广东省', '广西壮族自治区', '海南省',
                          '四川省', '贵州省', '云南省', '陕西省',
                          '甘肃省', '青海省', '西藏自治区', '宁夏回族自治区',
                          '新疆维吾尔自治区', '台湾省']

__cities: list[dict[str, str]] = [{'name': '直辖市',
                                   'cities': [
                                       '北京市',
                                       '上海市',
                                       '天津市',
                                       '重庆市']
                                   },
                                  {
                                      'name': '特别行政区',
                                      'cities': [
                                          '香港特别行政区',
                                          '澳门特别行政区']
                                  },
                                  {
                                      'name': '河北省',
                                      'cities': [
                                          '石家庄市',
                                          '唐山市',
                                          '秦皇岛市',
                                          '邯郸市',
                                          '邢台市',
                                          '保定市',
                                          '张家口市',
                                          '承德市',
                                          '沧州市',
                                          '廊坊市',
                                          '衡水市',
                                          '辛集市',
                                          '晋州市',
                                          '新乐市',
                                          '遵化市',
                                          '迁安市',
                                          '武安市',
                                          '南宫市',
                                          '沙河市',
                                          '涿州市',
                                          '定州市',
                                          '安国市',
                                          '高碑店市',
                                          '平泉市',
                                          '泊头市',
                                          '任丘市',
                                          '黄骅市',
                                          '河间市',
                                          '霸州市',
                                          '三河市',
                                          '深州市']
                                  },
                                  {
                                      'name': '山西省',
                                      'cities': [
                                          '太原市',
                                          '大同市',
                                          '阳泉市',
                                          '长治市',
                                          '晋城市',
                                          '朔州市',
                                          '晋中市',
                                          '运城市',
                                          '忻州市',
                                          '临汾市',
                                          '吕梁市',
                                          '古交市',
                                          '高平市',
                                          '介休市',
                                          '永济市',
                                          '河津市',
                                          '原平市',
                                          '侯马市',
                                          '霍州市',
                                          '孝义市',
                                          '汾阳市',
                                          '怀仁市']
                                  },
                                  {
                                      'name': '内蒙古自治区',
                                      'cities': [
                                          '呼和浩特市',
                                          '包头市',
                                          '乌海市',
                                          '赤峰市',
                                          '通辽市',
                                          '鄂尔多斯市',
                                          '呼伦贝尔市',
                                          '巴彦淖尔市',
                                          '乌兰察布市',
                                          '霍林郭勒市',
                                          '满洲里市',
                                          '牙克石市',
                                          '扎兰屯市',
                                          '额尔古纳市',
                                          '根河市',
                                          '丰镇市',
                                          '乌兰浩特市',
                                          '阿尔山市',
                                          '二连浩特市',
                                          '锡林浩特市']
                                  },
                                  {
                                      'name': '辽宁省',
                                      'cities': [
                                          '沈阳市',
                                          '大连市',
                                          '鞍山市',
                                          '抚顺市',
                                          '本溪市',
                                          '丹东市',
                                          '锦州市',
                                          '营口市',
                                          '阜新市',
                                          '辽阳市',
                                          '盘锦市',
                                          '铁岭市',
                                          '朝阳市',
                                          '葫芦岛市',
                                          '新民市',
                                          '瓦房店市',
                                          '庄河市',
                                          '海城市',
                                          '东港市',
                                          '凤城市',
                                          '凌海市',
                                          '北镇市',
                                          '盖州市',
                                          '大石桥市',
                                          '灯塔市',
                                          '调兵山市',
                                          '开原市',
                                          '北票市',
                                          '凌源市',
                                          '兴城市']
                                  },
                                  {
                                      'name': '吉林省',
                                      'cities': [
                                          '长春市',
                                          '吉林市',
                                          '四平市',
                                          '辽源市',
                                          '通化市',
                                          '白山市',
                                          '松原市',
                                          '白城市',
                                          '榆树市',
                                          '德惠市',
                                          '蛟河市',
                                          '桦甸市',
                                          '舒兰市',
                                          '磐石市',
                                          '公主岭市',
                                          '双辽市',
                                          '梅河口市',
                                          '集安市',
                                          '洮南市',
                                          '大安市',
                                          '临江市',
                                          '延吉市',
                                          '图们市',
                                          '敦化市',
                                          '珲春市',
                                          '龙井市',
                                          '和龙市',
                                          '扶余市']
                                  },
                                  {
                                      'name': '黑龙江省',
                                      'cities': [
                                          '哈尔滨市',
                                          '齐齐哈尔市',
                                          '黑河市',
                                          '大庆市',
                                          '伊春市',
                                          '鹤岗市',
                                          '佳木斯市',
                                          '双鸭山市',
                                          '七台河市',
                                          '鸡西市',
                                          '牡丹江市',
                                          '绥化市',
                                          '尚志市',
                                          '五常市',
                                          '讷河市',
                                          '北安市',
                                          '五大连池市',
                                          '嫩江市',
                                          '铁力市',
                                          '同江市',
                                          '富锦市',
                                          '虎林市',
                                          '密山市',
                                          '绥芬河市',
                                          '海林市',
                                          '宁安市',
                                          '安达市',
                                          '肇东市',
                                          '海伦市',
                                          '穆棱市',
                                          '东宁市',
                                          '抚远市',
                                          '漠河市']
                                  },
                                  {
                                      'name': '江苏省',
                                      'cities': [
                                          '南京市',
                                          '徐州市',
                                          '连云港市',
                                          '宿迁市',
                                          '淮安市',
                                          '盐城市',
                                          '扬州市',
                                          '泰州市',
                                          '南通市',
                                          '镇江市',
                                          '常州市',
                                          '无锡市',
                                          '苏州市',
                                          '常熟市',
                                          '张家港市',
                                          '太仓市',
                                          '昆山市',
                                          '江阴市',
                                          '宜兴市',
                                          '溧阳市',
                                          '扬中市',
                                          '句容市',
                                          '丹阳市',
                                          '如皋市',
                                          '启东市',
                                          '海安市',
                                          '高邮市',
                                          '仪征市',
                                          '兴化市',
                                          '泰兴市',
                                          '靖江市',
                                          '东台市',
                                          '邳州市',
                                          '新沂市']
                                  },
                                  {
                                      'name': '浙江省',
                                      'cities': [
                                          '杭州市',
                                          '宁波市',
                                          '湖州市',
                                          '嘉兴市',
                                          '舟山市',
                                          '绍兴市',
                                          '衢州市',
                                          '金华市',
                                          '台州市',
                                          '温州市',
                                          '丽水市',
                                          '建德市',
                                          '慈溪市',
                                          '余姚市',
                                          '平湖市',
                                          '海宁市',
                                          '桐乡市',
                                          '诸暨市',
                                          '嵊州市',
                                          '江山市',
                                          '兰溪市',
                                          '永康市',
                                          '义乌市',
                                          '东阳市',
                                          '临海市',
                                          '温岭市',
                                          '瑞安市',
                                          '乐清市',
                                          '龙港市',
                                          '龙泉市',
                                          '玉环市']
                                  },
                                  {
                                      'name': '安徽省',
                                      'cities': [
                                          '合肥市',
                                          '芜湖市',
                                          '蚌埠市',
                                          '淮南市',
                                          '马鞍山市',
                                          '淮北市',
                                          '铜陵市',
                                          '安庆市',
                                          '黄山市',
                                          '滁州市',
                                          '阜阳市',
                                          '宿州市',
                                          '六安市',
                                          '亳州市',
                                          '池州市',
                                          '宣城市',
                                          '巢湖市',
                                          '桐城市',
                                          '天长市',
                                          '明光市',
                                          '界首市',
                                          '宁国市',
                                          '广德市',
                                          '潜山市',
                                          '无为市']
                                  },
                                  {
                                      'name': '福建省',
                                      'cities': [
                                          '厦门市',
                                          '福州市',
                                          '南平市',
                                          '三明市',
                                          '莆田市',
                                          '泉州市',
                                          '漳州市',
                                          '龙岩市',
                                          '宁德市',
                                          '福清市',
                                          '邵武市',
                                          '武夷山市',
                                          '建瓯市',
                                          '永安市',
                                          '石狮市',
                                          '晋江市',
                                          '南安市',
                                          '龙海市',
                                          '漳平市',
                                          '福安市',
                                          '福鼎市']
                                  },
                                  {
                                      'name': '江西省',
                                      'cities': [
                                          '南昌市',
                                          '九江市',
                                          '景德镇市',
                                          '鹰潭市',
                                          '新余市',
                                          '萍乡市',
                                          '赣州市',
                                          '上饶市',
                                          '抚州市',
                                          '宜春市',
                                          '吉安市',
                                          '瑞昌市',
                                          '共青城市',
                                          '庐山市',
                                          '乐平市',
                                          '瑞金市',
                                          '德兴市',
                                          '丰城市',
                                          '樟树市',
                                          '高安市',
                                          '井冈山市',
                                          '贵溪市']
                                  },
                                  {
                                      'name': '山东省',
                                      'cities': [
                                          '济南市',
                                          '青岛市',
                                          '聊城市',
                                          '德州市',
                                          '东营市',
                                          '淄博市',
                                          '潍坊市',
                                          '烟台市',
                                          '威海市',
                                          '日照市',
                                          '临沂市',
                                          '枣庄市',
                                          '济宁市',
                                          '泰安市',
                                          '滨州市',
                                          '菏泽市',
                                          '胶州市',
                                          '平度市',
                                          '莱西市',
                                          '临清市',
                                          '乐陵市',
                                          '禹城市',
                                          '安丘市',
                                          '昌邑市',
                                          '高密市',
                                          '青州市',
                                          '诸城市',
                                          '寿光市',
                                          '栖霞市',
                                          '海阳市',
                                          '龙口市',
                                          '莱阳市',
                                          '莱州市',
                                          '蓬莱市',
                                          '招远市',
                                          '荣成市',
                                          '乳山市',
                                          '滕州市',
                                          '曲阜市',
                                          '邹城市',
                                          '新泰市',
                                          '肥城市',
                                          '邹平市']
                                  },
                                  {
                                      'name': '河南省',
                                      'cities': [
                                          '郑州市',
                                          '开封市',
                                          '洛阳市',
                                          '平顶山市',
                                          '安阳市',
                                          '鹤壁市',
                                          '新乡市',
                                          '焦作市',
                                          '濮阳市',
                                          '许昌市',
                                          '漯河市',
                                          '三门峡市',
                                          '南阳市',
                                          '商丘市',
                                          '周口市',
                                          '驻马店市',
                                          '信阳市',
                                          '荥阳市',
                                          '新郑市',
                                          '登封市',
                                          '新密市',
                                          '偃师市',
                                          '孟州市',
                                          '沁阳市',
                                          '卫辉市',
                                          '辉县市',
                                          '长垣市',
                                          '林州市',
                                          '禹州市',
                                          '长葛市',
                                          '舞钢市',
                                          '义马市',
                                          '灵宝市',
                                          '项城市',
                                          '巩义市',
                                          '邓州市',
                                          '永城市',
                                          '汝州市',
                                          '济源市']
                                  },
                                  {
                                      'name': '湖北省',
                                      'cities': [
                                          '武汉市',
                                          '十堰市',
                                          '襄阳市',
                                          '荆门市',
                                          '孝感市',
                                          '黄冈市',
                                          '鄂州市',
                                          '黄石市',
                                          '咸宁市',
                                          '荆州市',
                                          '宜昌市',
                                          '随州市',
                                          '丹江口市',
                                          '老河口市',
                                          '枣阳市',
                                          '宜城市',
                                          '钟祥市',
                                          '京山市',
                                          '汉川市',
                                          '应城市',
                                          '安陆市',
                                          '广水市',
                                          '麻城市',
                                          '武穴市',
                                          '大冶市',
                                          '赤壁市',
                                          '石首市',
                                          '洪湖市',
                                          '松滋市',
                                          '宜都市',
                                          '枝江市',
                                          '当阳市',
                                          '恩施市',
                                          '利川市',
                                          '仙桃市',
                                          '天门市',
                                          '潜江市']
                                  },
                                  {
                                      'name': '湖南省',
                                      'cities': [
                                          '长沙市',
                                          '衡阳市',
                                          '张家界市',
                                          '常德市',
                                          '益阳市',
                                          '岳阳市',
                                          '株洲市',
                                          '湘潭市',
                                          '郴州市',
                                          '永州市',
                                          '邵阳市',
                                          '怀化市',
                                          '娄底市',
                                          '耒阳市',
                                          '常宁市',
                                          '浏阳市',
                                          '津市市',
                                          '沅江市',
                                          '汨罗市',
                                          '临湘市',
                                          '醴陵市',
                                          '湘乡市',
                                          '韶山市',
                                          '资兴市',
                                          '武冈市',
                                          '邵东市',
                                          '洪江市',
                                          '冷水江市',
                                          '涟源市',
                                          '吉首市',
                                          '宁乡市']
                                  },
                                  {
                                      'name': '广东省',
                                      'cities': [
                                          '广州市',
                                          '深圳市',
                                          '清远市',
                                          '韶关市',
                                          '河源市',
                                          '梅州市',
                                          '潮州市',
                                          '汕头市',
                                          '揭阳市',
                                          '汕尾市',
                                          '惠州市',
                                          '东莞市',
                                          '珠海市',
                                          '中山市',
                                          '江门市',
                                          '佛山市',
                                          '肇庆市',
                                          '云浮市',
                                          '阳江市',
                                          '茂名市',
                                          '湛江市',
                                          '英德市',
                                          '连州市',
                                          '乐昌市',
                                          '南雄市',
                                          '兴宁市',
                                          '普宁市',
                                          '陆丰市',
                                          '恩平市',
                                          '台山市',
                                          '开平市',
                                          '鹤山市',
                                          '四会市',
                                          '罗定市',
                                          '阳春市',
                                          '化州市',
                                          '信宜市',
                                          '高州市',
                                          '吴川市',
                                          '廉江市',
                                          '雷州市']
                                  },
                                  {
                                      'name': '广西壮族自治区',
                                      'cities': ['南宁市',
                                                 '桂林市',
                                                 '柳州市',
                                                 '梧州市',
                                                 '贵港市',
                                                 '玉林市',
                                                 '钦州市',
                                                 '北海市',
                                                 '防城港市',
                                                 '崇左市',
                                                 '百色市',
                                                 '河池市',
                                                 '来宾市',
                                                 '贺州市',
                                                 '岑溪市',
                                                 '桂平市',
                                                 '北流市',
                                                 '东兴市',
                                                 '凭祥市',
                                                 '合山市',
                                                 '靖西市',
                                                 '平果市',
                                                 '荔浦市']
                                  },
                                  {
                                      'name': '海南省',
                                      'cities': ['海口市',
                                                 '三亚市',
                                                 '三沙市',
                                                 '儋州市',
                                                 '文昌市',
                                                 '琼海市',
                                                 '万宁市',
                                                 '东方市',
                                                 '五指山市']
                                  },
                                  {
                                      'name': '四川省',
                                      'cities': ['成都市',
                                                 '广元市',
                                                 '绵阳市',
                                                 '德阳市',
                                                 '南充市',
                                                 '广安市',
                                                 '遂宁市',
                                                 '内江市',
                                                 '乐山市',
                                                 '自贡市',
                                                 '泸州市',
                                                 '宜宾市',
                                                 '攀枝花市',
                                                 '巴中市',
                                                 '达州市',
                                                 '资阳市',
                                                 '眉山市',
                                                 '雅安市',
                                                 '崇州市',
                                                 '邛崃市',
                                                 '都江堰市',
                                                 '彭州市',
                                                 '江油市',
                                                 '什邡市',
                                                 '广汉市',
                                                 '绵竹市',
                                                 '阆中市',
                                                 '华蓥市',
                                                 '峨眉山市',
                                                 '万源市',
                                                 '简阳市',
                                                 '西昌市',
                                                 '康定市',
                                                 '马尔康市',
                                                 '隆昌市',
                                                 '射洪市',
                                                 '会理市']
                                  },
                                  {
                                      'name': '贵州省',
                                      'cities': ['贵阳市',
                                                 '六盘水市',
                                                 '遵义市',
                                                 '安顺市',
                                                 '毕节市',
                                                 '铜仁市',
                                                 '清镇市',
                                                 '赤水市',
                                                 '仁怀市',
                                                 '凯里市',
                                                 '都匀市',
                                                 '兴义市',
                                                 '福泉市',
                                                 '盘州市',
                                                 '兴仁市']
                                  },
                                  {
                                      'name': '云南省',
                                      'cities': ['昆明市',
                                                 '曲靖市',
                                                 '玉溪市',
                                                 '丽江市',
                                                 '昭通市',
                                                 '普洱市',
                                                 '临沧市',
                                                 '保山市',
                                                 '安宁市',
                                                 '宣威市',
                                                 '芒市',
                                                 '瑞丽市',
                                                 '大理市',
                                                 '楚雄市',
                                                 '个旧市',
                                                 '开远市',
                                                 '蒙自市',
                                                 '弥勒市',
                                                 '景洪市',
                                                 '文山市',
                                                 '香格里拉市',
                                                 '腾冲市',
                                                 '水富市',
                                                 '澄江市',
                                                 '泸水市']
                                  },
                                  {
                                      'name': '陕西省',
                                      'cities': ['西安市',
                                                 '延安市',
                                                 '铜川市',
                                                 '渭南市',
                                                 '咸阳市',
                                                 '宝鸡市',
                                                 '汉中市',
                                                 '榆林市',
                                                 '商洛市',
                                                 '安康市',
                                                 '韩城市',
                                                 '华阴市',
                                                 '兴平市',
                                                 '彬州市',
                                                 '神木市',
                                                 '子长市']
                                  },
                                  {
                                      'name': '甘肃省',
                                      'cities': ['兰州市',
                                                 '嘉峪关市',
                                                 '金昌市',
                                                 '白银市',
                                                 '天水市',
                                                 '酒泉市',
                                                 '张掖市',
                                                 '武威市',
                                                 '庆阳市',
                                                 '平凉市',
                                                 '定西市',
                                                 '陇南市',
                                                 '玉门市',
                                                 '敦煌市',
                                                 '临夏市',
                                                 '合作市',
                                                 '华亭市']
                                  },
                                  {
                                      'name': '青海省',
                                      'cities': ['西宁市',
                                                 '海东市',
                                                 '格尔木市',
                                                 '德令哈市',
                                                 '玉树市',
                                                 '茫崖市']
                                  },
                                  {
                                      'name': '西藏自治区',
                                      'cities': ['拉萨市',
                                                 '日喀则市',
                                                 '昌都市',
                                                 '林芝市',
                                                 '山南市',
                                                 '那曲市',
                                                 '阿里市']
                                  },
                                  {
                                      'name': '宁夏回族自治区',
                                      'cities': ['银川市',
                                                 '石嘴山市',
                                                 '吴忠市',
                                                 '中卫市',
                                                 '固原市',
                                                 '灵武市',
                                                 '青铜峡市']
                                  },
                                  {
                                      'name': '新疆维吾尔自治区',
                                      'cities': ['乌鲁木齐市',
                                                 '克拉玛依市',
                                                 '吐鲁番市',
                                                 '哈密市',
                                                 '喀什市',
                                                 '阿克苏市',
                                                 '库车市',
                                                 '和田市',
                                                 '阿图什市',
                                                 '阿拉山口市',
                                                 '博乐市',
                                                 '昌吉市',
                                                 '阜康市',
                                                 '库尔勒市',
                                                 '伊宁市',
                                                 '奎屯市',
                                                 '塔城市',
                                                 '乌苏市',
                                                 '阿勒泰市',
                                                 '霍尔果斯市',
                                                 '石河子市',
                                                 '阿拉尔市',
                                                 '图木舒克市',
                                                 '五家渠市',
                                                 '北屯市',
                                                 '铁门关市',
                                                 '双河市',
                                                 '可克达拉市',
                                                 '昆玉市',
                                                 '胡杨河市']
                                  },
                                  {
                                      'name': '台湾省',
                                      'cities': [
                                          '台北市',
                                          '新北市',
                                          '桃园市',
                                          '台中市',
                                          '台南市',
                                          '高雄市',
                                          '基隆市',
                                          '新竹市',
                                          '嘉义市']
                                  }]


def get_provinces():
    return __provinces


def get_cities(name: str):
    if not isinstance(name, str):
        raise TypeError('the argument(name) must be a string')
    for city in __cities:
        if city['name'] == name:
            return city
    raise ValueError('the argument(name) does not exist in the cities')



# @Coding: UTF-8
# @Time: 2025/3/26 23:42
# @Author: xieyang_ls
# @Filename: __init__.py.py

from pyutils_spirit.annotation import connection, get_instance_signature, singleton

from pyutils_spirit.component import components

from pyutils_spirit.concurrent_thread import (ReentryLock, Queue, LinkedQueue, BlockingQueue, SpiritID, Regional,
                                              ThreadExecutor, ThreadHolder)

from pyutils_spirit.util import (Assemble,
                                 HashAssemble,
                                 deep_dumps,
                                 deep_loads,
                                 get_provinces,
                                 get_cities,
                                 Set,
                                 HashSet)

from pyutils_spirit.database import Handler, MySQLHandler

from pyutils_spirit.exception import (ArgumentException, ConflictSignatureError,
                                      NoneSignatureError, BusinessException,
                                      SystemException)

from pyutils_spirit.python_spark import PySparkHandler

from pyutils_spirit.style import (BLACK, RED, GREEN, YELLOW,
                                  BLUE, MAGENTA, CYAN, WHITE, RESET,
                                  set_spirit_banner,
                                  set_websocket_banner)

from pyutils_spirit.spirit_container import (get, post, put, delete,
                                             exception_advice, throws_exception,
                                             request_interceptor,
                                             interceptor_before, interceptor_after,
                                             Result, SpiritApplication,
                                             component, mapper,
                                             service, controller,
                                             resource, MultipartFile)

from pyutils_spirit.tcp import websocket_server, endpoint, Session, onopen, onmessage, onclose, onerror

__all__ = ['connection',
           'get_instance_signature',
           'singleton',
           'components',
           'ReentryLock',
           'Regional',
           'SpiritID',
           'ThreadExecutor',
           'Assemble',
           'HashAssemble',
           'deep_dumps',
           'deep_loads',
           'get_provinces',
           'get_cities',
           'Queue',
           'LinkedQueue',
           'BlockingQueue',
           'Set',
           'HashSet',
           'Handler',
           'MySQLHandler',
           'ArgumentException',
           'ConflictSignatureError',
           'NoneSignatureError',
           'PySparkHandler',
           "BLACK",
           "RED",
           "GREEN",
           "YELLOW",
           "BLUE",
           "MAGENTA",
           "CYAN",
           "WHITE",
           "RESET",
           "set_spirit_banner",
           "set_websocket_banner",
           "get",
           "post",
           "put",
           "delete",
           "exception_advice",
           "throws_exception",
           "request_interceptor",
           "interceptor_before",
           "interceptor_after",
           "SpiritApplication",
           "component",
           "mapper",
           "service",
           "controller",
           "resource",
           'websocket_server',
           'endpoint',
           'Session',
           'onopen',
           'onmessage',
           'onclose',
           'onerror',
           'ThreadHolder',
           'BusinessException',
           'SystemException',
           'PySparkHandler',
           'Result',
           'MultipartFile']



# @Coding: UTF-8
# @Time: 2024/9/11 0:15
# @Author: xieyang_ls
# @Filename: mysql_handler.py
from abc import ABC, abstractmethod

from logging import info, error, INFO, basicConfig

from pyutils_spirit.util.json_util import deep_loads

from pymysql import Connect, OperationalError, cursors

from pyutils_spirit.annotation.interpreter import singleton

from pyutils_spirit.exception.exception import ArgumentException

basicConfig(level=INFO)


class Handler(ABC):

    @abstractmethod
    def select(self, sql: str) -> tuple:
        pass

    @abstractmethod
    def insert(self, sql: str) -> bool:
        pass

    @abstractmethod
    def update(self, sql: str) -> bool:
        pass

    @abstractmethod
    def delete(self, sql: str) -> bool:
        pass

    @abstractmethod
    def manage_execute(self, sql: str) -> bool:
        pass

    @abstractmethod
    def disconnect(self) -> None:
        pass

    @abstractmethod
    def get_connection(self) -> [Connect]:
        pass


@singleton(signature="MysqlHandler")
class MySQLHandler(Handler):
    __cursor = None

    __connection = None

    def __init__(self, host: str, port: int, user: str, password: str, database: str) -> None:
        try:
            connection: Connect = Connect(
                host=host,
                port=port,
                user=user,
                password=password,
                cursorclass=cursors.DictCursor
            )
            connection.select_db(database)
            info(f"Connected to database {database} successfully!!!")
            info(f"MySQL version: {connection.get_server_info()}")
            cursor = connection.cursor()
            self.__connection = connection
            self.__cursor = cursor
        except (ArgumentException, OperationalError) as e:
            error(e)
            error(f"Connected to database {database} failure")
            raise ArgumentException("please check connected the database arguments")

    def select(self, sql: str) -> tuple:
        self.__cursor.execute(sql)
        result: tuple = self.__cursor.fetchall()
        return deep_loads(data=result)

    def insert(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def update(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def delete(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def manage_execute(self, sql: str) -> bool:
        try:
            self.__cursor.execute(sql)
            self.__connection.commit()
            return True
        except Exception as e:
            self.__connection.rollback()
            raise e

    def get_connection(self) -> [Connect]:
        return self.__connection, self.__cursor

    def disconnect(self) -> None:
        self.__connection.close()



# @Coding: UTF-8
# @Time: 2024/9/18 21:15
# @Author: xieyang_ls
# @Filename: set.py
import time
from abc import ABC, abstractmethod
from threading import Thread

from typing import TypeVar, Generic

E = TypeVar('E')


class Set(ABC, Generic[E]):

    @abstractmethod
    def __init__(self) -> None:
        raise NotImplementedError

    @abstractmethod
    def add(self, element: E) -> None:
        pass

    @abstractmethod
    def remove(self, element: E) -> E:
        pass

    @abstractmethod
    def clean(self) -> None:
        pass

    @abstractmethod
    def __contains__(self, element: E) -> bool:
        pass

    @abstractmethod
    def __len__(self) -> int:
        pass


class HashSet(Set[E]):
    __LOAD_FACTOR = 0.75

    def __init__(self, initial_capacity: int = 8) -> None:
        if not isinstance(initial_capacity, int) or initial_capacity < 1:
            self.__initial_capacity: int = 8
        else:
            self.__initial_capacity: int = initial_capacity
        self.__nodes: list[HashSet._Node[E] | None] = [None] * self.__initial_capacity
        self.__nodes_size: int = 0

    def __get_hash_id_code(self, element: E) -> int:
        hash_id_code: int | None = None
        try:
            hash_id_code = hash(element)
        except TypeError:
            hash_id_code = id(element)
        finally:
            return abs(hash_id_code) % self.__initial_capacity

    def __auto_expansion(self) -> None:
        self.__initial_capacity *= 2
        new_nodes: list[HashSet._Node[E] | None] = [None] * self.__initial_capacity
        for node in self.__nodes:
            while node is not None:
                index: int = self.__get_hash_id_code(node.element)
                if new_nodes[index] is None:
                    new_nodes[index] = node
                else:
                    new_nodes[index].add_element_tail(e_node=node)
                node = node.get_different_hash_id_node(index=index, get_hash_id_code=self.__get_hash_id_code)
        self.__nodes = new_nodes

    def add(self, element: E) -> None:
        if self.__nodes_size / self.__initial_capacity >= HashSet.__LOAD_FACTOR:
            self.__auto_expansion()
        index: int = self.__get_hash_id_code(element=element)
        if self.__nodes[index] is None:
            self.__nodes[index] = HashSet._Node(element=element)
            self.__nodes_size += 1
        elif self.__nodes[index].element is element or self.__nodes[index].element == element:
            return None
        else:
            if self.__nodes[index].add_element(element=element):
                self.__nodes_size += 1

    def remove(self, element: E) -> E:
        index: int = self.__get_hash_id_code(element=element)
        if self.__nodes[index] is None:
            return None
        if self.__nodes[index].element is element or self.__nodes[index].element == element:
            r_element: HashSet._Node = self.__nodes[index].element
            self.__nodes[index] = self.__nodes[index].next
            self.__nodes_size -= 1
            return r_element
        r_element: HashSet._Node = self.__nodes[index].remove_element(element=element)
        if r_element is not None:
            self.__nodes_size -= 1
        return r_element

    def clean(self) -> None:
        self.__initial_capacity = 8
        self.__nodes_size = 0
        self.__nodes = [None] * self.__initial_capacity

    def __contains__(self, element: E) -> bool:
        index: int = self.__get_hash_id_code(element=element)
        if self.__nodes[index] is None:
            return False
        elif self.__nodes[index].element is element or self.__nodes[index].element == element:
            return True
        return self.__nodes[index].contains_element(element=element)

    def __len__(self) -> int:
        return self.__nodes_size

    class _Node(Generic[E]):

        def __init__(self, element: E) -> None:
            self.element: E = element
            self.next: HashSet._Node[E] | None = None

        def add_element(self, element: E) -> bool:
            if self.next is None:
                self.next = HashSet._Node(element=element)
                return True
            elif self.next.element is element or self.next.element == element:
                return False
            return self.next.add_element(element=element)

        def remove_element(self, element: E) -> E:
            if self.next is None:
                return None
            if self.next.element is element or self.next.element == element:
                r_element: HashSet._Node = self.next.element
                self.next = self.next.next
                return r_element
            return self.next.remove_element(element=element)

        def add_element_tail(self, e_node) -> None:
            if self.next is None:
                self.next = e_node
            else:
                return self.next.add_element_tail(e_node=e_node)

        def get_different_hash_id_node(self, index: int, get_hash_id_code: callable):
            if self.next is None:
                return None
            other_index: int = get_hash_id_code(element=self.next.element)
            if other_index == index:
                return self.next.get_different_hash_id_node(index=index, get_hash_id_code=get_hash_id_code)
            e_node: HashSet._Node = self.next
            self.next = None
            return e_node

        def contains_element(self, element: E) -> bool:
            if self.next is None:
                return False
            if self.next.element is element or self.next.element == element:
                return True
            return self.next.contains_element(element=element)
